declare module "electron-publisher-s3/out/uploader" {
  /// <reference types="node" />
  import { S3 } from "aws-sdk"
  import { CreateMultipartUploadRequest } from "aws-sdk/clients/s3"
  import { EventEmitter } from "events"
  import { Stats } from "fs-extra-p"

  export class S3Client {
    readonly s3: S3
    readonly s3RetryCount: number
    readonly s3RetryDelay: number
    readonly multipartUploadThreshold: number
    readonly multipartUploadSize: number
    readonly multipartDownloadThreshold: number
    readonly multipartDownloadSize: number
    constructor(options?: any)
  }

  export class Uploader extends EventEmitter {
    private readonly client
    private readonly s3Options
    private readonly localFile
    /** @readonly */
    loaded: number
    private cancelled
    readonly contentLength: number
    constructor(client: S3Client, s3Options: CreateMultipartUploadRequest, localFile: string, localFileStat: Stats)
    upload(): Promise<void>
    abort(): void
    private putObject(md5)
    private multipartUpload(uploadId, multipartUploadSize)
    private makeUploadPart(part, uploadId)
    private runOrRetry<T>(task)
  }
}

declare module "electron-publisher-s3" {
  import { S3Options } from "electron-builder-http/out/publishOptions"
  import { PublishContext, Publisher } from "electron-publish"

  export default class S3Publisher extends Publisher {
    private readonly info
    readonly providerName: string
    constructor(context: PublishContext, info: S3Options)
    static checkAndResolveOptions(options: S3Options, channelFromAppVersion: string | null): Promise<void>
    upload(file: string, safeArtifactName?: string): Promise<any>
    toString(): string
  }
}

